"""Synchronized Lyrics providers"""

from .netease import NetEase
from .deezer import Deezer
from .lyricsify import Lyricsify
from .megalobiz import Megalobiz
from .musixmatch import Musixmatch
from .lrclib import Lrclib
from .tencent import Tencent
